package weaver.interfaces.hansang.job;


import com.alibaba.fastjson.JSON;
import com.engine.kq.util.KQDurationCalculatorUtil;
import org.apache.commons.lang.StringUtils;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.formmode.setup.ModeRightInfo;
import weaver.general.TimeUtil;
import weaver.general.Util;
import weaver.interfaces.schedule.BaseCronJob;

import java.time.LocalTime;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * @author wanxq
 * @date 2023年08月03日11:15:39
 * @desc 实习生出勤天数计算
 */
public class InternAttDaySynJob extends BaseCronJob {
    private String salaryMonth;

    public String getSalaryMonth() {
        return salaryMonth;
    }

    public void setSalaryMonth(String salaryMonth) {
        this.salaryMonth = salaryMonth;
    }


    @Override
    public void execute() {
        RecordSet rs = new RecordSet();
        String fromDate = DateUtil.getFirstDayOfMonthToString();
        String toDate = DateUtil.getLastDayOfMonthToString();

        if(StringUtils.isNotBlank(salaryMonth)){
            fromDate = DateUtil.getFirstDayOfMonth(this.salaryMonth+"-01");
            toDate = DateUtil.getLastDayOfMonth(this.salaryMonth+"-01");
        }

        String sql = " select id,field35 from cus_fielddata cf where field24 in(2,3) ";
        rs.execute(sql);
        while(rs.next()){
            int userid = Util.getIntValue(rs.getString("id"));
            int userAttr = Util.getIntValue(rs.getString("field35"));
            calAttendance(userid,userAttr,fromDate,toDate);
        }
    }

    /**
     * 计算人员出勤天数
     * @param resourceid
     * @param fromDate
     * @param toDate
     */
    public void calAttendance(int resourceid,int userAttr,String fromDate,String toDate){
        double attendanceDays = 0;
        Map<String,String> data = getSignInfo(resourceid,fromDate,toDate);

        RecordSet rs = new RecordSet();
        String sql = "  select kqdate,workbegindate,workbegintime ,workenddate ,workendtime,workmins, attendanceMins  " +
                " from kq_format_detail kfd " +
                " where resourceid =? and kqdate>=? and kqdate<=?  order by kqdate ";
        rs.executeQuery(sql,resourceid,fromDate,toDate);
        while(rs.next()){
            String kqdate = Util.null2String(rs.getString("kqdate"));
            double workmins =  rs.getDouble("workmins")<0?0:rs.getDouble("workmins");
            double attendanceMins = rs.getDouble("attendanceMins")<0?0:rs.getDouble("attendanceMins");
            double attendanceHours =Util.getDoubleValue(KQDurationCalculatorUtil.getDurationRound(attendanceMins/(60.0)+""));

            //办公加班月结的员工，周六周日按原逻辑计算，不算在此规则中
            if(userAttr==0 && (DateUtil.getWeek(kqdate) == 6 || DateUtil.getWeek(kqdate) == 7)){
                continue;
            }

            double currentAttDays = 0;
            if(data.get(kqdate)!=null && !"".equals(data.get(kqdate))){
                //取当天最早的打卡，判断是上午卡还是下午卡
                String signBeginTime = data.get(kqdate).split("\\|")[0];
                try{
                    LocalTime localTime1 = LocalTime.parse(signBeginTime);
                    LocalTime localTime2 = LocalTime.parse("12:00:00");

                    int comparison = localTime1.compareTo(localTime2);
                    if (comparison < 0) {
                        //上午打卡，当天工作时长大于等于8小时算1天，大于等于3.5小时算0.5天
                        if(attendanceHours>=8){
                            currentAttDays = 1;
                        }else if(attendanceHours>=3.5){
                            currentAttDays = 0.5;
                        }

                    } else {
                        //下午打卡工作时长必须大于等于4.5小时算0.5天
                        if(attendanceHours>=4.5){
                            currentAttDays = 0.5;
                        }
                    }
                    rs.writeLog("InternAttDaySynJob calAttendance kqdate==>"+kqdate+" currentAttDays==>"+currentAttDays);
                    attendanceDays += currentAttDays;
                }catch (Exception e){
                    e.printStackTrace();
                }
            }
        }
        //更新实习人员出勤表
        String month = fromDate.substring(0,7);
        insertData(resourceid,attendanceDays,month);
    }


    public void insertData(int userid,double workdays,String month){
        RecordSet rs = new RecordSet();
        //删除建模表中当前算薪月的数据
        rs.executeUpdate("delete from uf_sxskqsc where sxsry=? and xzszy=? ",userid,month);

        int formmodeid = getModeIdByName("实习生考勤时长");
        String modeuuid = UUID.randomUUID().toString();
        int modedatacreater = 1;
        String modedatacreatedate = TimeUtil.getCurrentDateString();
        String modedatacreatetime = TimeUtil.getOnlyCurrentTimeString();
        String insertSql = " insert into uf_sxskqsc(sxsry,sjcqts,xzszy,formmodeid,modedatacreater,modedatacreatedate," +
                "modedatacreatetime,modeuuid) values(?,?,?,?,?,?,?,?) ";
        rs.executeUpdate(insertSql,userid,workdays,month,
                formmodeid,modedatacreater,modedatacreatedate,modedatacreatetime,modeuuid);

        rs.executeQuery("select id from uf_sxskqsc where modeuuid=?",modeuuid);
        rs.next();
        int id = rs.getInt("id");
        if(id>0){
            ModeRightInfo right = new ModeRightInfo();
            right.editModeDataShare(userid, formmodeid, id);
        }

    }


    /**
     * 根据建模名称获取建模id,建模权限同步的使用
     * @param modeName 建模名称
     * @return int 建模id
     */
    public static int getModeIdByName(String modeName){
        RecordSet rs = new RecordSet();
        String sql = " SELECT ID FROM MODEINFO WHERE MODENAME = ? ";
        rs.executeQuery(sql, modeName);
        if (rs.next()) {
            return rs.getInt("id");
        }
        return -1;
    }



    /**
     * 查询对应考勤人员每天的打卡数据
     * @param userid
     * @param fromDate
     * @param toDate
     * @return
     */
    public Map<String,String> getSignInfo(int userid, String fromDate, String toDate){
        Map<String,String> data = new HashMap<>();
        RecordSet rs = new RecordSet();
        String sql = " select signdate,min(signtime) signbegintime,max(signtime) signendtime from HrmScheduleSign " +
                " where userid=? and signdate>=? and signdate<=? and signtime>='06:30:00' " +
                " group by signdate ";
        rs.executeQuery(sql,userid,fromDate,toDate);
        while (rs.next()){
            String signDate = Util.null2String(rs.getString("signdate"));
            String signBeginTime = Util.null2String(rs.getString("signbegintime"));
            String signEndTime = Util.null2String(rs.getString("signendtime"));
            data.put(signDate,signBeginTime+"|"+signEndTime);
        }
        rs.writeLog("InternAttDaySynJob getSignInfo data==>"+ JSON.toJSONString(data));
        return data;
    }




}
