package weaver.interfaces.hansang.job;


import com.engine.kq.biz.KQHolidaySetBiz;
import com.engine.kq.biz.KQWorkTime;
import com.weaver.formmodel.util.DateHelper;
import org.apache.commons.lang.StringUtils;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.formmode.setup.ModeRightInfo;
import weaver.general.BaseBean;
import weaver.general.TimeUtil;
import weaver.general.Util;
import weaver.interfaces.schedule.BaseCronJob;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.UUID;

/**
 * @author wanxq
 * @date 2023年07月24日14:32:09
 * @desc
 */
public class LeaveDaysCalculateJob extends BaseCronJob {
    private BaseBean log = new BaseBean();

    private String salaryMonth;


    public void setMonth(String salaryMonth) {
        this.salaryMonth = salaryMonth;
    }

    @Override
    public void execute() {
        if(StringUtils.isBlank(salaryMonth)){
            salaryMonth = DateHelper.getCurrentMonth();
        }

        KQWorkTime kqWorkTime = new KQWorkTime();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        RecordSet rs = new RecordSet();

        String sql = " select id,field82 from cus_fielddata " +
                "where scopeid=3 and scope='HrmCustomFieldByInfoType' and field82 like '"+salaryMonth+"%' ";
        rs.executeQuery(sql);
        while(rs.next()){
            int userid = rs.getInt("id");
            String field82 = Util.null2String(rs.getString("field82"));
            LocalDate leaveDate = LocalDate.parse(field82,formatter);

            log.writeLog("LeaveDaysCalculateJob leave userid==>"+userid+" leaveDate==>"+leaveDate);

            //判断是法定节假日前还是后
            //int holidayDays = getHolidayDays(salaryMonth,leaveDate);
            //log.writeLog("LeaveDaysCalculateJob leave holidayDays==>"+holidayDays);

            double workdays = 0;
            LocalDate workStartDate = LocalDate.parse(salaryMonth+"-01",formatter);
            long daysDiff = ChronoUnit.DAYS.between(workStartDate, leaveDate);
            for(int i=0;i<daysDiff;i++) {
                String tempdate = workStartDate.plusDays(i).toString();

                int changeType = KQHolidaySetBiz.getChangeType("2",tempdate);
                boolean isWeekend = DateUtil.getWeek(tempdate) == 6 || DateUtil.getWeek(tempdate) == 7;

                log.writeLog("LeaveDaysCalculateJob userid==>"+userid+" tempdate==>"+tempdate+" changeType==>"+changeType+" isWeekend==>"+isWeekend);

                if(changeType==-1 && !isWeekend){
                    workdays ++;
                    log.writeLog("LeaveDaysCalculateJob userid==>"+userid+" workday==>"+tempdate);
                }

                if(changeType==1 || changeType==2){
                    workdays ++;
                    log.writeLog("LeaveDaysCalculateJob userid==>"+userid+" workday==>"+tempdate);
                }


                //判断是否是法定节假日、周末
                //boolean isHoliday = false;
                //int changeType = KQHolidaySetBiz.getChangeType("2",tempdate);
                //log.writeLog("LeaveDaysCalculateJob userid==>"+userid+" tempdate==>"+tempdate+" changeType==>"+changeType+" weekend==>"+DateUtil.getWeek(tempdate));
                //if (changeType != 2 && (DateUtil.getWeek(tempdate) == 6 || DateUtil.getWeek(tempdate) == 7)) {
                //    isHoliday = true;
                //}
                //if (changeType == 1 || changeType == 3) {
                //    isHoliday = true;
                //}
                //
                //if(!isHoliday){
                //    workdays++;
                //    log.writeLog("LeaveDaysCalculateJob userid==>"+userid+" workday==>"+tempdate);
                //}
            }
            log.writeLog("LeaveDaysCalculateJob userid==>"+userid+" allworkdays==>"+workdays);
            //workdays += holidayDays;

            insertData(userid,workdays);
        }

    }

    public void insertData(int userid,double workdays){
        RecordSet rs = new RecordSet();
        //删除建模表中当前算薪月的数据
        rs.executeUpdate("delete from uf_lzkk where xzszy=? and ryid=? ",salaryMonth,userid);

        int formmodeid = getModeIdByName("离职扣款");
        String modeuuid = UUID.randomUUID().toString();
        int modedatacreater = userid;
        String modedatacreatedate = TimeUtil.getCurrentDateString();
        String modedatacreatetime = TimeUtil.getOnlyCurrentTimeString();
        String insertSql = " insert into uf_lzkk(ryid,lzkk,xzszy,formmodeid,modedatacreater,modedatacreatedate," +
                "modedatacreatetime,modeuuid) values(?,?,?,?,?,?,?,?) ";
        rs.executeUpdate(insertSql,userid,workdays,salaryMonth,
                formmodeid,modedatacreater,modedatacreatedate,modedatacreatetime,modeuuid);

        rs.executeQuery("select id from uf_lzkk where modeuuid=?",modeuuid);
        rs.next();
        int id = rs.getInt("id");
        if(id>0){
            ModeRightInfo right = new ModeRightInfo();
            right.editModeDataShare(userid, formmodeid, id);
        }

    }

    /**
     * 获取离职日期前的节假日天数
     * @param salaryMonth
     * @param leaveDate
     * @return
     */
    public int getHolidayDays(String salaryMonth,LocalDate leaveDate){
        int holidayDays = 0;
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        RecordSet rs = new RecordSet();
        String sql = " select holidaydate from kq_HolidaySet where groupid=1 and changetype=1 and holidaydate like '"+salaryMonth+"%' ";
        rs.execute(sql);
        while (rs.next()){
            LocalDate holidaydate = LocalDate.parse(Util.null2String(rs.getString("holidaydate")),formatter);

            if(leaveDate.isAfter(holidaydate)){
                holidayDays++;
                log.writeLog("leaveDate==>"+leaveDate+" holidaydate ==>"+holidaydate);
            }
        }
        return holidayDays;
    }


    /**
     * 根据建模名称获取建模id,建模权限同步的使用
     * @param modeName 建模名称
     * @return int 建模id
     */
    public static int getModeIdByName(String modeName){
        RecordSet rs = new RecordSet();
        String sql = " SELECT ID FROM MODEINFO WHERE MODENAME = ? ";
        rs.executeQuery(sql, modeName);
        if (rs.next()) {
            return rs.getInt("id");
        }
        return -1;
    }



}
