package weaver.interfaces.leshanvc.cronjob;

import com.engine.common.util.ServiceUtil;
import com.engine.hrm.service.impl.OrganizationServiceImpl;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.hrm.User;
import weaver.interfaces.schedule.BaseCronJob;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 自动启用、封存部门计划任务
 *
 * @author:dxfeng
 * @createTime: 2023/03/06
 * @version: 1.0
 */
public class AutoCancelDepartmentJob extends BaseCronJob {
    @Override
    public void execute() {
        User user = new User(1);
        String currentDate = DateUtil.getCurrentDate();
        List<String> disableList = new ArrayList<>();
        List<String> enableList = new ArrayList<>();
        RecordSet rs = new RecordSet();
        // 查询待封存的部门ID
        String sql = "select deptid from hrmdepartmentdefined where fcrq = ? ";
        rs.executeQuery(sql, currentDate);
        while (rs.next()) {
            disableList.add(rs.getString("deptid"));
        }

        // 查询待启用的部门ID
        sql = "select deptid from hrmdepartmentdefined where qyrq = ? ";
        rs.executeQuery(sql, currentDate);
        while (rs.next()) {
            enableList.add(rs.getString("deptid"));
        }


        // 批量禁用部门
        if (CollectionUtils.isNotEmpty(disableList)) {
            Map<String, Object> params = new HashMap<>();
            params.put("id", StringUtils.join(disableList, ","));
            Map<String, Object> result = ServiceUtil.getService(OrganizationServiceImpl.class, user).doDepartmentCancel(params, user);
            new BaseBean().writeLog(result);
        }

        // 批量启用部门
        if (CollectionUtils.isNotEmpty(enableList)) {
            Map<String, Object> params = new HashMap<>();
            params.put("id", StringUtils.join(enableList, ","));
            Map<String, Object> result = ServiceUtil.getService(OrganizationServiceImpl.class, user).doDepartmentISCanceled(params, user);
            new BaseBean().writeLog(result);
        }
    }

}
